﻿using System;
using System.Threading.Tasks;
using GraphQL.Client;
using GraphQL.Common.Request;
using GraphQL.Common.Response;

namespace GraphQLFailRepro
{
    class Program
    {
        /// <summary>
        /// API request that works.
        /// </summary>
        static async Task<GraphQLResponse> AttemptSwapiIntrospection()
        {
            var graphClient = new GraphQLClient("https://swapi.apis.guru/");
            return await graphClient.GetIntrospectionQueryAsync();
        }

        /// <summary>
        /// API request that fails.
        /// </summary>
        static async Task<GraphQLResponse> AttemptGitHubIntrospection()
        {
            var graphClient = new GraphQLClient("https://api.github.com/graphql");
            return await graphClient.GetIntrospectionQueryAsync();
        }

        /// <summary>
        /// API request I would actually like to make (also fails).
        /// </summary>
        static async Task<GraphQLResponse> AttemptGitHubQuery(string gitHubAccessToken)
        {
            // Sample from GitHub API docs: https://developer.github.com/v4/guides/forming-calls/#communicating-with-graphql
            var graphClient = new GraphQLClient("https://api.github.com/graphql");
            graphClient.DefaultRequestHeaders.Add("Authorization", $"bearer {gitHubAccessToken}");
            return await graphClient.GetQueryAsync(@"query { viewer { login } }");
        }

        static async Task Main(string[] args)
        {
            try
            {
                var x = await AttemptSwapiIntrospection();
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }

            try
            {
                var y = await AttemptGitHubIntrospection();
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }

            //try
            //{
            //    var z = await AttemptGitHubQuery("some-github-access-token-here");
            //}
            //catch (Exception ex)
            //{
            //    Console.WriteLine(ex);
            //}

            Console.Read();
        }
    }
}
